set long 4096
-- XMLAGG Test
 select sys_xmlagg(SYS_XMLGEN(Ename)).getClobVal() emps
  		from emp
  		where deptno = 10;
--
pause
--
-- XMLELEMENT + XMLATTRIBUTES + XMLAGG
--
select xmlelement("employee",
          xmlagg(xmlelement("emp",xmlattributes(empno as "empno",ename as "ename"),
                  xmlelement("job",job),
                  xmlelement("hiredate",hiredate),
                  xmlelement("pay",xmlattributes(nvl(sal,0) as "sal",nvl(comm,0) as "comm"))))) myxml
from emp;
--
pause
--
-- Silly nested queries
--
select ename,job,sal,
  (select avg(sal) from emp where job = main.job) jobavgsal
from emp main;

select ename,sal from emp
  where sal between (select avg(sal) from emp where job = 'SALESMAN')
                          and (select avg(sal) from emp where job = 'ANALYST');

select deptno from dept 
  where (select avg(sal) from emp) > 
    (select avg(sal) from emp
     	where emp.deptno = dept.deptno);
     	
select ename,sal from emp
  order by (select dname from dept where dept.deptno = emp.deptno),ename;
--
pause
-- RANK
select dname,
       nvl(avg(sal),0) avg_sal,
       count(empno) nbr_emps,
       rank() over (order by nvl(avg(sal),0)) rank
   from emp right join dept using (deptno)
group by dname;
--
--
-- RANK LAST
select dname,
       min(salary) keep (dense_rank first order by hire_date) "First Hired",
       max(salary) keep (dense_rank last order by hire_date) "Last Hired"
   from hr.employees right join dept
         on department_id = deptno
group by dname;
--
pause
-- 
-- GROUPING SETS
select deptno
      ,job
      ,count(empno) nbremps
      ,sum(sal)     totpay
    from emp
    group by grouping sets (deptno,job,(deptno,job));
--
pause
-- ORDER SIBLINGS BY
--
col empid format a20
select lpad(' ',level*2)||empno empid,ename,mgr,deptno,level
     from emp
     connect by prior empno = mgr
     start with mgr is null
     order siblings by (ename);
--
pause
-- TIMES
select sys_extract_utc(to_timestamp_tz(current_timestamp)) 
from dual;

select substr(tzname,1,25) tzname
      ,substr(tzabbrev,1,25) tzabbrev
 from v$timezone_names;
 
select tz_offset('US/Eastern') from dual;
--
pause
-- MERGE
merge
    into bonus
    using emp
    on ( bonus.ename = emp.ename )
    when matched
    then  update  -- only one update match allowed!
           set bonus.sal = emp.sal, 
           bonus.comm = emp.comm
    when not matched
        then insert
           (ename,job,sal,comm)
          values
           (emp.ename,emp.job,emp.sal,emp.comm);
rollback;
--
-- EXTERNAL TABLES
pause
--
drop directory demo_source;
create directory demo_source as 'C:\Classes2006\Conferences\IOUGCOLLAB2006\Ora10g9iDiffs';
--
pause
--
drop table newemp;
create table newemp
  ( empno      number(4) 
   ,ename      char(10)
   ,job        char(9)
   ,mgr        number(4)
   ,hiredate   date
   ,sal        number(7,2)
   ,comm       number(7,2)
   ,deptno     number(2)
  )
  organization external
  (type oracle_loader 
    default directory demo_source
    access parameters
    ( 
     records delimited by newline 
     fields terminated by ',' 
     missing field values are null 
     ( empno, ename, job, mgr, 
        hiredate char date_format date mask "mm-dd-yyyy", 
        sal, comm, deptno 
      ) 
    ) 
   location ('personc.dat')
  );
--
desc newemp
select * from newemp;
--
pause
-- INSERT ALL
insert all
  into emp 
     (empno,ename,job,mgr,hiredate,sal,comm,deptno)
    values 
     (empno,ename,job,mgr,hiredate,sal,comm,deptno)
  into bonus 
     (ename,job,sal,comm)
    values
     (ename,job,sal,comm)
  select empno,ename,job,mgr,hiredate,sal,comm,deptno
         from newemp;
  rollback;
--
-- INSERT CONDITIONAL
insert first
   when job = 'SALESMAN' then
     into emp  
       (empno,ename,job,mgr,hiredate,sal,comm,deptno)
        values
       (empno,ename,job,mgr,hiredate,sal,comm,deptno)
     into bonus (ename,job,sal,comm)
         values (ename,job,sal,comm)
   else
     into emp
       (empno,ename,job,mgr,hiredate,sal,comm,deptno)
     values
       (empno,ename,job,mgr,hiredate,sal,comm,deptno)
  select empno,ename,job,mgr,hiredate,sal,comm,deptno
         from newemp;
  rollback;
--
pause
--*********************************************************************
--*** Code to test Old and New join types                           ***
--*********************************************************************
--*** Separate into four scripts, some code duplicated              ***
--*********************************************************************
--*********************************************************************
--*** Test older inner join                                         ***
--*********************************************************************
  select distinct nvl(dname,'No Dept'),count(empno) nbr_emps
     from many_emps emp,many_depts dept
     where emp.deptno = dept.deptno
       and emp.job in ('MANAGER','SALESMAN','ANALYST')
     group by dname;
--*********************************************************************
--*** Test newer ANSI inner Join                                    ***
--*********************************************************************
  select distinct nvl(dname,'No Dept'),count(empno) nbr_emps
    from many_emps emp join many_depts dept
      on emp.deptno = dept.deptno
    where emp.job in ('MANAGER','SALESMAN','ANALYST')
    group by dname;
pause
--*********************************************************************
--*** Test older, Oracle Outer Join Operator                        ***
--*********************************************************************
  select distinct nvl(dname,'No Dept'),count(empno) nbr_emps
     from many_emps emp,many_depts dept
     where emp.deptno(+) = dept.deptno
     group by dname;
--*********************************************************************
--*** Test newer, ANSI Right Join                                   ***
--*********************************************************************
  select distinct nvl(dname,'No Dept'),count(empno) nbr_emps
    from many_emps emp right join many_depts dept
      on emp.deptno = dept.deptno
     group by dname;
pause
--*********************************************************************
--*** Test outer join using union                                   ***
--*********************************************************************
  select nvl(dname,'No Dept') deptname,count(empno) nbr_emps
     from many_emps emp,many_depts dept
     where emp.deptno(+) = dept.deptno
     group by dname
  union
  select nvl(dname,'No Dept') deptname,count(empno) nbr_emps
     from many_emps emp,many_depts dept
     where emp.deptno = dept.deptno(+)
     group by dname;
--*********************************************************************
--*** Test newer, ANSI Full Join                                    ***
--*********************************************************************
  select distinct nvl(dname,'No Dept') deptname,count(empno) nbr_emps
    from many_emps emp full join many_depts dept
      on emp.deptno = dept.deptno
     group by dname;
--
pause
--
-- CASE
--
select ename,sal,
    case when job = 'CLERK' then 'GLUE'
         when job = 'MANAGER' then 'SUPER'
         else job end job_x 
    from emp 
       where case when sal < 1000 
                    then sal + 2000
                  when sal < 2000 
                    then sal + 1000
                  else sal
              end    > 2900
    order by case when sal < 1000 
                      then sal + 9000
                  when sal < 2000 
                       then sal + 7000
                  else sal
             end;
pause
select ename,sal,
       case job when 'CLERK' then 'Producer'
                when 'ANALYST' then 'Producer'
                when 'PRESIDENT' then 'Overhead'
                when 'SALESMAN' then 'Producer'
                else 'Overhead'
       end emptype
   from emp;
pause
select country,corr(sale,cnt)
from sales_view
group by country;
pause
select avg(sal) avg
      ,median(sal) median
      ,deptno
   from many_emps
   group by deptno;
pause
drop table myjunktable;
create table myjunktable
as select * from emp;
select count(*) from myjunktable;
drop table myjunktable;
select count(*) from myjunktable;
show recyclebin;
flashback table myjunktable to before drop;
select count(*) from myjunktable;
drop table myjunktable purge;
purge recyclebin;
pause
select prod_id
     , substr(prod_name,1,20) prod_name
     , substr(prod_desc,1,30) prod_desc
   from sh.products
   where prod_name like ('E%')
      or prod_name like ('P%')
   order by prod_id;
select prod_id
     , substr(prod_name,1,20) prod_name
     , substr(prod_desc,1,30) prod_desc
   from sh.products
   where regexp_like (prod_name,'^E|^P')
   order by prod_id;
pause
SELECT SUBSTR(country,1,20) country, SUBSTR(prod,1,15) prod, year, sales
 FROM sales_view
 MODEL RETURN UPDATED ROWS
  PARTITION BY (country)
  DIMENSION BY (prod, year)
  MEASURES (sale sales)
  RULES (
   sales['Mouse Pad', 2002] = sales['Mouse Pad', 2001] +
        sales['Mouse Pad', 2000])
 ORDER BY country, prod, year;
 pause
 commit;
 update bonus set sal = 1000 where ename = 'ALLEN';
 update emp set sal = 4000 where ename = 'JONES';
 select e.ename,e.sal,b.ename,b.sal
  from emp e join bonus b
    on e.ename = b.ename;
 merge
     into bonus
     using emp
     on ( bonus.ename = emp.ename )
     when matched
     then  update -- only one update match allowed!
            set bonus.sal = emp.sal,
                bonus.comm = emp.comm
           delete
            where bonus.sal > 3999
     when not matched
         then insert
            (ename,job,sal,comm)
           values
            (emp.ename,emp.job,emp.sal,emp.comm);
 select e.ename,e.sal,b.ename,b.sal
  from emp e join bonus b
    on e.ename = b.ename;
rollback;
pause
select count(*) nbr_rows
  from sh.sales;
select count(*) * 10 nbr_rows
  from sh.sales sample(10);
select count(*) * 10 nbr_rows
  from sh.sales sample(10);
select count(*) * 10 nbr_rows
  from sh.sales sample(10) seed(1);
select count(*) * 10 nbr_rows
  from sh.sales sample(10) seed(1);